package be.iminds.ilabt.jfed.ui.javafx.automated_testing_gui;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.ui.javafx.choosers.ServerChooser;
import be.iminds.ilabt.jfed.util.JFedUtils;
import javafx.beans.property.*;
import javafx.scene.control.SelectionModel;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;

/**
 * AutomatedTesterModel
 */
public class AutomatedTesterModel {
    private static final Logger LOG = LoggerFactory.getLogger(AutomatedTesterModel.class);

    @Nonnull private final SelectionModel<String> classNameSelectionModel;
    @Nonnull private final SelectionModel<String> testGroupSelectionModel;
    @Nonnull private final ServerChooser serverChooser;
    @Nonnull private final BooleanProperty showRawModel;

    @Nullable private final File propertiesFile;

    //auto save only enables AFTER restore is complete
    private boolean autosaveEnabled = false;


    public AutomatedTesterModel(@Nonnull SelectionModel<String> classNameSelectionModel,
                                @Nonnull SelectionModel<String> testGroupSelectionModel,
                                @Nonnull ServerChooser serverChooser,
                                @Nonnull BooleanProperty showRawModel) {
        autosaveEnabled = false;
        this.classNameSelectionModel = classNameSelectionModel;
        this.testGroupSelectionModel = testGroupSelectionModel;
        this.serverChooser = serverChooser;
        this.showRawModel = showRawModel;

        classNameSelectionModel.selectedItemProperty().addListener((p, o, n) -> autoSave());
        testGroupSelectionModel.selectedItemProperty().addListener((p, o, n) -> autoSave());
        serverChooser.selectedAuthorityProperty().addListener((p, o, n) -> autoSave());
        showRawModel.addListener((p, o, n) -> autoSave());

        this.propertiesFile = JFedUtils.createPropertiesFile("automated-tester.properties");
    }

    private PropertiesConfiguration cachedConf = null;
    private PropertiesConfiguration getPropertiesConfiguration() {
        if (cachedConf != null)
            return cachedConf;
        if (propertiesFile == null) {
            System.err.println("Cannot load AutomatedTester properties");
            return null;
        }

        PropertiesConfiguration conf;
        try {
            conf = new PropertiesConfiguration(propertiesFile);
        } catch (ConfigurationException e) {
            System.err.println("Cannot restore AutomatedTester properties: "+e.getMessage());
            LOG.error("Error restoring AutomatedTester config", e);
            return null;
        }
        conf.setThrowExceptionOnMissing(false);
        cachedConf = conf;
        return conf;
    }

    public void restore() {
        autosaveEnabled = false;

        PropertiesConfiguration conf = getPropertiesConfiguration();
        if (conf == null) {
            return;
        }

        Integer serverId = conf.getInteger("server_id", null);
        if (serverId != null)
            serverChooser.select(serverId);

        String className = conf.getString("class_name", null);
        if (className == null)
            classNameSelectionModel.clearSelection();
        else
            classNameSelectionModel.select(className);

        String testGroup = conf.getString("test_group", null);
        if (testGroup == null)
            testGroupSelectionModel.clearSelection();
        else
            testGroupSelectionModel.select(testGroup);

        boolean showRaw = conf.getBoolean("show_raw", false);
        showRawModel.setValue(showRaw);

        autosaveEnabled = true;
    }
    public void autoSave() {
        if (autosaveEnabled) save();
    }
    public void save() {
        PropertiesConfiguration conf = getPropertiesConfiguration();
        if (conf == null) {
            return;
        }

        if (classNameSelectionModel.isEmpty())
            conf.clearProperty("class_name");
        else
            conf.setProperty("class_name", classNameSelectionModel.getSelectedItem());

        if (testGroupSelectionModel.isEmpty())
            conf.clearProperty("test_group");
        else
            conf.setProperty("test_group", testGroupSelectionModel.getSelectedItem());

        if (serverChooser.getSelectedServer() == null)
            conf.clearProperty("server_id");
        else
            conf.setProperty("server_id", serverChooser.getSelectedServer().getId());

        conf.setProperty("show_raw", showRawModel.get());

        try {
            conf.save();
        } catch (ConfigurationException e) {
            System.err.println("Cannot saving AutomatedTester properties: "+e.getMessage());
            LOG.error("Error saving AutomatedTester config", e);
        }
    }

    public String getSelectedClassName() {
        return classNameSelectionModel.selectedItemProperty().get();
    }
    public ReadOnlyObjectProperty<String> selectedClassNameProperty() {
        return classNameSelectionModel.selectedItemProperty();
    }
    public void setSelectedClassName(String selectedClassName) {
        this.classNameSelectionModel.select(selectedClassName);
    }


    public String getSelectedTestGroup() {
        return testGroupSelectionModel.selectedItemProperty().get();
    }
    public ReadOnlyObjectProperty<String> selectedTestGroupProperty() {
        return testGroupSelectionModel.selectedItemProperty();
    }
    public void setSelectedTestGroup(String selectedTestGroup) {
        this.testGroupSelectionModel.select(selectedTestGroup);
    }


    public Server getSelectedServer() {
        return serverChooser.selectedAuthorityProperty().get();
    }
    public Integer getSelectedServerId() {
        Server res = getSelectedServer();
        if (res == null)
            return null;
        return res.getId();
    }
    public ReadOnlyObjectProperty<Server> selectedSelectedServerProperty() {
        return serverChooser.selectedAuthorityProperty();
    }
    public void setSelectedServer(Server server) {
        this.serverChooser.select(server);
    }
    public void setSelectedServer(int serverId) {
        this.serverChooser.select(serverId);
    }


    @Nonnull
    public boolean isShowRawModel() {
        return showRawModel.get();
    }
    @Nonnull
    public BooleanProperty showRawModelProperty() {
        return showRawModel;
    }
    public boolean toggleShowRaw() {
        boolean newValue = !showRawModel.get();
        showRawModel.setValue(newValue);
        return newValue;
    }
}
