package be.iminds.ilabt.jfed.experimenter_gui.tasks;

import be.iminds.ilabt.jfed.espec.bundle.BundleFetcher;
import be.iminds.ilabt.jfed.espec.bundle.ESpecBundle;
import be.iminds.ilabt.jfed.espec.bundle.LocalDirESpecBundle;
import be.iminds.ilabt.jfed.espec.model.ExperimentSpecification;
import be.iminds.ilabt.jfed.espec.parser.ExperimentSpecificationParser;
import be.iminds.ilabt.jfed.experimenter_gui.dialogs.DialogsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.events.OpenESpecEvent;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.SfaModel;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.lowlevel.TestbedInfoSource;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import com.google.common.eventbus.EventBus;
import javafx.application.Platform;
import javafx.concurrent.Task;
import javafx.stage.Stage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.io.IOException;

/**
 * Recovers an existing slice. The method to do this is chosen automatically.
 * <p/>
 * This method might immediately return, starting the recovery in the background.
 */
public final class OpenESpecTask extends Task<ESpecBundle> {
    private static final Logger LOG = LoggerFactory.getLogger(OpenESpecTask.class);

    //private final ExperimenterGUI experimenterGUI;
    private final Stage ownerStage = null;
    @Nonnull  private final OpenESpecEvent openESpecEvent;

    private final DialogsFactory dialogsFactory;


    OpenESpecTask(@Nonnull OpenESpecEvent openESpecEvent,
                  DialogsFactory dialogsFactory) {

        this.openESpecEvent = openESpecEvent;
        this.dialogsFactory = dialogsFactory;

        updateTitle("Fetching ExperimentSpecification '" + openESpecEvent + "'");

    }

    @Override
    protected ESpecBundle call() {
        try {
            ESpecBundle eSpecBundle;
            switch (openESpecEvent.getESpecSource()) {
                case DIR: {
                    if (openESpecEvent.getFile() == null)
                        throw new NullPointerException("openESpecEvent.getFile() is not supposed to be null when openESpecEvent.getESpecSource()="+openESpecEvent.getESpecSource());
                    eSpecBundle = BundleFetcher.fetchDir(openESpecEvent.getFile());
                    break;
                }
                case ARCHIVE: {
                    if (openESpecEvent.getFile() == null)
                        throw new NullPointerException("openESpecEvent.getFile() is not supposed to be null when openESpecEvent.getESpecSource()="+openESpecEvent.getESpecSource());
                    eSpecBundle = BundleFetcher.fetchArchiveFile(openESpecEvent.getFile());
                    break;
                }
                case URL: {
                    if (openESpecEvent.getUrl() == null)
                        throw new NullPointerException("openESpecEvent.getUrl() is not supposed to be null when openESpecEvent.getESpecSource()="+openESpecEvent.getESpecSource());
                    eSpecBundle = BundleFetcher.fetchArchiveUrl(openESpecEvent.getUrl());
                    break;
                }
                default: throw new RuntimeException("ESpecSource "+openESpecEvent.getESpecSource()+" is not supported");
            }
            ExperimentSpecification eSpec = eSpecBundle.getExperimentSpecification();
            return eSpecBundle;
        } catch (InterruptedException e) {
            LOG.error("Got interrupted while fetching ESpecBundle", e);
            return null;
        }  catch (IllegalArgumentException e) {
            if (e.getCause() != null &&
                    e.getCause() instanceof ExperimentSpecificationParser.ExperimentSpecificationParseException) {
                ExperimentSpecificationParser.ExperimentSpecificationParseException cause =
                        (ExperimentSpecificationParser.ExperimentSpecificationParseException) e.getCause();
                Platform.runLater(() ->
                        JFDialogs.create()
                                .owner(ownerStage)
                                .title("Parsing ExperimentSpecification failed")
                                .masthead("Parsing ExperimentSpecification failed")
                                .message("Error: " + cause.getMessage())
                                .showError()
                );
                LOG.debug("Showed parse error to user", cause);
                return null;
            }
            if (e.getCause() != null &&
                    e.getCause() instanceof IOException) {
                IOException cause =
                        (IOException) e.getCause();
                Platform.runLater(() ->
                        JFDialogs.create()
                                .owner(ownerStage)
                                .title("Opening ExperimentSpecification failed")
                                .masthead("Opening ExperimentSpecification failed")
                                .message("IO Error: " + cause.getMessage())
                                .showError()
                );
                return null;
            }
            Platform.runLater(() ->
                    JFDialogs.create()
                            .owner(ownerStage)
                            .title("Opening ExperimentSpecification failed")
                            .masthead("Opening ExperimentSpecification failed")
                            .message("An error occurred while opening ExperimentSpecification: " + e.getMessage())
                            .showException(e)
            );
            LOG.error("Something is wrong with the ESpecBundle", e);
            return null;
        } catch (ESpecBundle.ESpecBundleInitException e) {
            LOG.error("Got an error while fetching ESpecBundle", e);
            Platform.runLater(() ->
                    JFDialogs.create()
                            .owner(ownerStage)
                            .title("Opening ExperimentSpecification failed")
                            .masthead("Opening ExperimentSpecification failed")
                            .message("An error occurred while opening ExperimentSpecification: " + e.getMessage())
                            .showException(e)
            );
            return null;
        }
    }

    @Override
    protected void failed() {
        LOG.warn("An error occurred while opening ExperimentSpecification: " + getMessage(), getException());
        JFDialogs.create()
                .owner(ownerStage)
                .title("Opening ExperimentSpecification failed")
                .masthead("Opening ExperimentSpecification failed")
                .message("An error occurred while opening ExperimentSpecification: " + getMessage())
                .showException(getException());

        super.failed();
    }

    @Override
    protected void succeeded() {
        LOG.info("Successfully opened ExperimentSpecification Bundle");
        ESpecBundle eSpecBundle = this.getValue();
        if (eSpecBundle != null) {
            dialogsFactory.showCreateSliceDialog(null, eSpecBundle);
        }
        //if null, an error occured but was handled
    }
}
